@@ -25,12 +25,6 @@
 #include <math.h>
 #include <signal.h>
 
-int opt_verbose = 0;
-int opt_debug = 0;
-int opt_debug_json = 0;
-int opt_batch_mode = 0;
-int opt_progress_frequency = 0;
-
 /* interrupt handling */
 volatile int quit = 0;
 static int signals_blocked = 0;
@@ -77,38 +71,17 @@ void check_signal(int *r)
 		*r = -EINTR;
 }
 
-#define LOG_MAX_LEN 4096
-
-__attribute__((format(printf, 5, 6)))
-void clogger(struct crypt_device *cd, int level, const char *file, int line,
-	     const char *format, ...)
+void tool_log(int level, const char *msg, void *usrptr)
 {
-	va_list argp;
-	char target[LOG_MAX_LEN + 2];
+	struct tools_log_params *params = (struct tools_log_params *)usrptr;
 
-	va_start(argp, format);
-
-	if (vsnprintf(&target[0], LOG_MAX_LEN, format, argp) > 0) {
-		/* All verbose and error messages in tools end with EOL. */
-		if (level == CRYPT_LOG_VERBOSE || level == CRYPT_LOG_ERROR ||
-		    level == CRYPT_LOG_DEBUG || level == CRYPT_LOG_DEBUG_JSON)
-			strncat(target, "\n", LOG_MAX_LEN);
-
-		crypt_log(cd, level, target);
-	}
-
-	va_end(argp);
-}
-
-void tool_log(int level, const char *msg, void *usrptr __attribute__((unused)))
-{
-	switch(level) {
+	switch (level) {
 
 	case CRYPT_LOG_NORMAL:
 		fprintf(stdout, "%s", msg);
 		break;
 	case CRYPT_LOG_VERBOSE:
-		if (opt_verbose)
+		if (params && params->verbose)
 			fprintf(stdout, "%s", msg);
 		break;
 	case CRYPT_LOG_ERROR:
@@ -116,7 +89,7 @@ void tool_log(int level, const char *msg, void *usrptr __attribute__((unused)))
 		break;
 	case CRYPT_LOG_DEBUG_JSON:
 	case CRYPT_LOG_DEBUG:
-		if (opt_debug)
+		if (params && params->debug)
 			fprintf(stdout, "# %s", msg);
 		break;
 	}
@@ -124,8 +97,10 @@ void tool_log(int level, const char *msg, void *usrptr __attribute__((unused)))
 
 void quiet_log(int level, const char *msg, void *usrptr)
 {
-	if (!opt_verbose && (level == CRYPT_LOG_ERROR || level == CRYPT_LOG_NORMAL))
-		level = CRYPT_LOG_VERBOSE;
+	struct tools_log_params *params = (struct tools_log_params *)usrptr;
+
+	if ((!params || !params->verbose) && (level == CRYPT_LOG_ERROR || level == CRYPT_LOG_NORMAL))
+		return;
 	tool_log(level, msg, usrptr);
 }
 
@@ -140,7 +115,7 @@ static int _dialog(const char *msg, void *usrptr, int default_answer)
 	if (block)
 		set_int_block(0);
 
-	if (isatty(STDIN_FILENO) && !opt_batch_mode) {
+	if (isatty(STDIN_FILENO)) {
 		log_std("\nWARNING!\n========\n");
 		log_std("%s\n\nAre you sure? (Type 'yes' in capital letters): ", msg);
 		fflush(stdout);
@@ -179,11 +154,8 @@ void show_status(int errcode)
 {
 	char *crypt_error;
 
-	if(!opt_verbose)
-		return;
-
-	if(!errcode) {
-		log_std(_("Command successful.\n"));
+	if (!errcode) {
+		log_verbose(_("Command successful."));
 		return;
 	}
 
@@ -203,7 +175,7 @@ void show_status(int errcode)
 	else
 		crypt_error = _("unknown error");
 
-	log_std(_("Command failed with code %i (%s).\n"), -errcode, crypt_error);
+	log_verbose(_("Command failed with code %i (%s)."), -errcode, crypt_error);
 }
 
 const char *uuid_or_device(const char *spec)
@@ -299,6 +271,36 @@ void tools_token_msg(int token, crypt_object_op op)
 		log_verbose(_("Token %i removed."), token);
 }
 
+void tools_token_error_msg(int error, const char *type, int token, bool pin_provided)
+{
+	if (error >= 0)
+		return;
+
+	if (error == -ENOANO) {
+		if (pin_provided)
+			log_verbose(_("No token could be unlocked with this PIN."));
+		else if (token != CRYPT_ANY_TOKEN)
+			log_verbose(_("Token %i requires PIN."), token);
+		else if (type)
+			log_verbose(_("Token (type %s) requires PIN."), type);
+	} else if (error == -EPERM) {
+		if (token != CRYPT_ANY_TOKEN)
+			log_verbose(_("Token %i cannot unlock assigned keyslot(s) (wrong keyslot passphrase)."), token);
+		else if (type)
+			log_verbose(_("Token (type %s) cannot unlock assigned keyslot(s) (wrong keyslot passphrase)."), type);
+	} if (error == -EAGAIN) {
+		if (token != CRYPT_ANY_TOKEN)
+			log_verbose(_("Token %i requires additional missing resource."), token);
+		else if (type)
+			log_verbose(_("Token (type %s) requires additional missing resource."), type);
+	} if (error == -ENOENT) {
+		if (type)
+			log_verbose(_("No usable token (type %s) is available."), type);
+		else
+			log_verbose(_("No usable token is available."));
+	}
+}
+
 /*
  * Device size string parsing, suffixes:
  * s|S - 512 bytes sectors
@@ -306,7 +308,7 @@ void tools_token_msg(int token, crypt_object_op op)
  * kiB|KiB|miB|MiB|giB|GiB|tiB|TiB - 1024 base
  * kb |KB |mM |MB |gB |GB |tB |TB  - 1000 base
  */
-int tools_string_to_size(struct crypt_device *cd, const char *s, uint64_t *size)
+int tools_string_to_size(const char *s, uint64_t *size)
 {
 	char *endp = NULL;
 	size_t len;
@@ -373,16 +375,13 @@ static double time_diff(struct timeval *start, struct timeval *end)
 		+ (end->tv_usec - start->tv_usec) / 1E6;
 }
 
-void tools_clear_line(void)
+static void tools_clear_line(void)
 {
-	if (opt_progress_frequency)
-		return;
 	/* vt100 code clear line */
 	log_std("\33[2K\r");
 }
 
-static void tools_time_progress(uint64_t device_size, uint64_t bytes, uint64_t *start_bytes,
-			 struct timeval *start_time, struct timeval *end_time)
+static void tools_time_progress(uint64_t device_size, uint64_t bytes, struct tools_progress_params *parms)
 {
 	struct timeval now_time;
 	unsigned long long mbytes, eta;
@@ -390,38 +389,34 @@ static void tools_time_progress(uint64_t device_size, uint64_t bytes, uint64_t *
 	int final = (bytes == device_size);
 	const char *eol, *ustr = "";
 
-	if (opt_batch_mode)
-		return;
-
 	gettimeofday(&now_time, NULL);
-	if (start_time->tv_sec == 0 && start_time->tv_usec == 0) {
-		*start_time = now_time;
-		*end_time = now_time;
-		*start_bytes = bytes;
+	if (parms->start_time.tv_sec == 0 && parms->start_time.tv_usec == 0) {
+		parms->start_time = now_time;
+		parms->end_time = now_time;
+		parms->start_offset = bytes;
 		return;
 	}
 
-	if (opt_progress_frequency) {
-		frequency = (double)opt_progress_frequency;
+	if (parms->frequency) {
+		frequency = (double)parms->frequency;
 		eol = "\n";
 	} else {
 		frequency = 0.5;
 		eol = "";
 	}
 
-	if (!final && time_diff(end_time, &now_time) < frequency)
+	if (!final && time_diff(&parms->end_time, &now_time) < frequency)
 		return;
 
-	*end_time = now_time;
+	parms->end_time = now_time;
 
-	tdiff = time_diff(start_time, end_time);
+	tdiff = time_diff(&parms->start_time, &parms->end_time);
 	if (!tdiff)
 		return;
 
 	mbytes = bytes  / 1024 / 1024;
-	uib = (double)(bytes - *start_bytes) / tdiff;
+	uib = (double)(bytes - parms->start_offset) / tdiff;
 
-	/* FIXME: calculate this from last minute only. */
 	eta = (unsigned long long)(device_size / uib - tdiff);
 
 	if (uib > 1073741824.0f) {
@@ -435,7 +430,8 @@ static void tools_time_progress(uint64_t device_size, uint64_t bytes, uint64_t *
 		ustr = "Ki";
 	}
 
-	tools_clear_line();
+	if (!parms->frequency)
+		tools_clear_line();
 	if (final)
 		log_std("Finished, time %02llu:%02llu.%03llu, "
 			"%4llu MiB written, speed %5.1f %sB/s\n",
@@ -453,150 +449,22 @@ static void tools_time_progress(uint64_t device_size, uint64_t bytes, uint64_t *
 
 int tools_wipe_progress(uint64_t size, uint64_t offset, void *usrptr)
 {
-	static struct timeval start_time = {}, end_time = {};
-	static uint64_t start_offset = 0;
 	int r = 0;
+	struct tools_progress_params *parms = (struct tools_progress_params *)usrptr;
 
-	tools_time_progress(size, offset, &start_offset, &start_time, &end_time);
+	if (parms && !parms->batch_mode)
+		tools_time_progress(size, offset, parms);
 
 	check_signal(&r);
 	if (r) {
-		tools_clear_line();
+		if (!parms || !parms->frequency)
+			tools_clear_line();
 		log_err(_("\nWipe interrupted."));
 	}
 
 	return r;
 }
 
-static void report_partition(const char *value, const char *device)
-{
-	if (opt_batch_mode)
-		log_dbg("Device %s already contains a '%s' partition signature.", device, value);
-	else
-		log_std(_("WARNING: Device %s already contains a '%s' partition signature.\n"), device, value);
-}
-
-static void report_superblock(const char *value, const char *device)
-{
-	if (opt_batch_mode)
-		log_dbg("Device %s already contains a '%s' superblock signature.", device, value);
-	else
-		log_std(_("WARNING: Device %s already contains a '%s' superblock signature.\n"), device, value);
-}
-
-int tools_detect_signatures(const char *device, int ignore_luks, size_t *count)
-{
-	int r;
-	size_t tmp_count;
-	struct blkid_handle *h;
-	blk_probe_status pr;
-
-	if (!count)
-		count = &tmp_count;
-
-	*count = 0;
-
-	if (!blk_supported()) {
-		log_dbg("Blkid support disabled.");
-		return 0;
-	}
-
-	if ((r = blk_init_by_path(&h, device))) {
-		log_err(_("Failed to initialize device signature probes."));
-		return -EINVAL;
-	}
-
-	blk_set_chains_for_full_print(h);
-
-	if (ignore_luks && blk_superblocks_filter_luks(h)) {
-		r = -EINVAL;
-		goto out;
-	}
-
-	while ((pr = blk_probe(h)) < PRB_EMPTY) {
-		if (blk_is_partition(h))
-			report_partition(blk_get_partition_type(h), device);
-		else if (blk_is_superblock(h))
-			report_superblock(blk_get_superblock_type(h), device);
-		else {
-			log_dbg("Internal tools_detect_signatures() error.");
-			r = -EINVAL;
-			goto out;
-		}
-		(*count)++;
-	}
-
-	if (pr == PRB_FAIL)
-		r = -EINVAL;
-out:
-	blk_free(h);
-	return r;
-}
-
-int tools_wipe_all_signatures(const char *path)
-{
-	int fd, flags, r;
-	blk_probe_status pr;
-	struct stat st;
-	struct blkid_handle *h = NULL;
-
-	if (!blk_supported()) {
-		log_dbg("Blkid support disabled.");
-		return 0;
-	}
-
-	if (stat(path, &st)) {
-		log_err(_("Failed to stat device %s."), path);
-		return -EINVAL;
-	}
-
-	flags = O_RDWR;
-	if (S_ISBLK(st.st_mode))
-		flags |= O_EXCL;
-
-	/* better than opening regular file with O_EXCL (undefined) */
-	/* coverity[toctou] */
-	fd = open(path, flags);
-	if (fd < 0) {
-		if (errno == EBUSY)
-			log_err(_("Device %s is in use. Can not proceed with format operation."), path);
-		else
-			log_err(_("Failed to open file %s in read/write mode."), path);
-		return -EINVAL;
-	}
-
-	if ((r = blk_init_by_fd(&h, fd))) {
-		log_err(_("Failed to initialize device signature probes."));
-		r = -EINVAL;
-		goto out;
-	}
-
-	blk_set_chains_for_wipes(h);
-
-	while ((pr = blk_probe(h)) < PRB_EMPTY) {
-		if (blk_is_partition(h))
-			log_verbose(_("Existing '%s' partition signature (offset: %" PRIi64 " bytes) on device %s will be wiped."),
-				    blk_get_partition_type(h), blk_get_offset(h), path);
-		if (blk_is_superblock(h))
-			log_verbose(_("Existing '%s' superblock signature (offset: %" PRIi64 " bytes) on device %s will be wiped."),
-				    blk_get_superblock_type(h), blk_get_offset(h), path);
-		if (blk_do_wipe(h)) {
-			log_err(_("Failed to wipe device signature."));
-			r = -EINVAL;
-			goto out;
-		}
-	}
-
-	if (pr != PRB_EMPTY) {
-		log_err(_("Failed to probe device %s for a signature."), path);
-		r = -EINVAL;
-	}
-out:
-	close(fd);
-	blk_free(h);
-	return r;
-}
-
 /*
  * Keyfile - is standard input treated as a binary file (no EOL handling).
  */
@@ -610,15 +478,16 @@ int tools_is_stdin(const char *key_file)
 
 int tools_reencrypt_progress(uint64_t size, uint64_t offset, void *usrptr)
 {
-	static struct timeval start_time = {}, end_time = {};
-	static uint64_t start_offset = 0;
 	int r = 0;
+	struct tools_progress_params *parms = (struct tools_progress_params *)usrptr;
 
-	tools_time_progress(size, offset, &start_offset, &start_time, &end_time);
+	if (parms && !parms->batch_mode)
+		tools_time_progress(size, offset, parms);
 
 	check_signal(&r);
 	if (r) {
-		tools_clear_line();
+		if (!parms || !parms->frequency)
+			tools_clear_line();
 		log_err(_("\nReencryption interrupted."));
 	}
 
